@@ -116,9 +116,6 @@ static gboolean      entry_focus_out              (YelpLocationEntry  *entry,
                                                    GdkEventFocus      *event,
                                                    YelpWindow         *window);
 
-static void          view_external_uri            (YelpView           *view,
-                                                   YelpUri            *uri,
-                                                   YelpWindow         *window);
 static void          view_new_window              (YelpView           *view,
                                                    YelpUri            *uri,
                                                    YelpWindow         *window);
@@ -225,6 +222,7 @@ struct _YelpWindowPrivate {
     GtkWidget    *bookmarks_list;
     GtkListStore *bookmarks_store;
     gulong        bookmarks_changed;
+    gulong        read_later_changed;
 
     /* no refs on these, owned by containers */
     YelpView *view;
@@ -368,6 +366,11 @@ yelp_window_dispose (GObject *object)
         priv->bookmark_actions = NULL;
     }
 
+    if (priv->read_later_changed) {
+        g_source_remove (priv->read_later_changed);
+        priv->read_later_changed = 0;
+    }
+
     if (priv->bookmarks_changed) {
         g_source_remove (priv->bookmarks_changed);
         priv->bookmarks_changed = 0;
@@ -468,7 +471,9 @@ window_construct (YelpWindow *window)
     yelp_view_add_link_action (priv->view, action,
                                (YelpViewActionValidFunc) view_is_xref_uri,
                                window);
-    g_signal_connect (priv->application, "read-later-changed", G_CALLBACK (app_read_later_changed), window);
+    priv->read_later_changed =
+        g_signal_connect (priv->application, "read-later-changed",
+                          G_CALLBACK (app_read_later_changed), window);
 
     priv->vbox_full = gtk_vbox_new (FALSE, 3);
     gtk_container_add (GTK_CONTAINER (window), priv->vbox_full);
@@ -589,7 +594,6 @@ window_construct (YelpWindow *window)
     priv->read_later_vbox = gtk_vbox_new (FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), priv->read_later_vbox, FALSE, FALSE, 0);
 
-    g_signal_connect (priv->view, "external-uri", G_CALLBACK (view_external_uri), window);
     g_signal_connect (priv->view, "new-view-requested", G_CALLBACK (view_new_window), window);
     g_signal_connect (priv->view, "loaded", G_CALLBACK (view_loaded), window);
     g_signal_connect (priv->view, "notify::yelp-uri", G_CALLBACK (view_uri_selected), window);
@@ -1420,23 +1424,6 @@ entry_focus_out (YelpLocationEntry  *entry,
     return FALSE;
 }
 
-static void
-view_external_uri (YelpView   *view,
-                   YelpUri    *uri,
-                   YelpWindow *window)
-{
-    gchar *struri = yelp_uri_get_canonical_uri (uri);
-    if (g_str_has_prefix (struri, "install:")) {
-        YelpWindowPrivate *priv = GET_PRIV (window);
-        gchar *pkg = struri + 8;
-        yelp_application_install_package (priv->application, pkg, "");
-    }
-    else
-        g_app_info_launch_default_for_uri (struri, NULL, NULL);
-
-    g_free (struri);
-}
-
 static void
 view_new_window (YelpView   *view,
                  YelpUri    *uri,
@@ -1452,27 +1439,40 @@ view_loaded (YelpView   *view,
 {
 
     YelpUri *uri;
-    gchar *doc_uri, *page_id, *icon, *title;
+    gchar *doc_uri;
+    YelpViewState state;
+    GdkWindow *gdkwin;
     YelpWindowPrivate *priv = GET_PRIV (window);
 
     g_object_get (view,
                   "yelp-uri", &uri,
-                  "page-id", &page_id,
-                  "page-icon", &icon,
-                  "page-title", &title,
+                  "state", &state,
                   NULL);
     doc_uri = yelp_uri_get_document_uri (uri);
-    yelp_application_update_bookmarks (priv->application,
-                                       doc_uri,
-                                       page_id,
-                                       icon,
-                                       title);
-    g_free (page_id);
-    g_free (icon);
-    g_free (title);
 
-    app_read_later_changed (priv->application, doc_uri, window);
+    gdkwin = gtk_widget_get_window (GTK_WIDGET (window));
+    if (gdkwin != NULL)
+        gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (window)), NULL);
 
+    if (state != YELP_VIEW_STATE_ERROR) {
+        gchar *page_id, *icon, *title;
+        g_object_get (view,
+                      "page-id", &page_id,
+                      "page-icon", &icon,
+                      "page-title", &title,
+                      NULL);
+        yelp_application_update_bookmarks (priv->application,
+                                           doc_uri,
+                                           page_id,
+                                           icon,
+                                           title);
+        app_read_later_changed (priv->application, doc_uri, window);
+        g_free (page_id);
+        g_free (icon);
+        g_free (title);
+    }
+
+    g_free (doc_uri);
     g_object_unref (uri);
 }
 
@@ -1483,12 +1483,19 @@ view_uri_selected (YelpView     *view,
 {
     YelpUri *uri;
     gchar *doc_uri;
+    GdkWindow *gdkwin;
     YelpWindowPrivate *priv = GET_PRIV (window);
 
     g_object_get (G_OBJECT (view), "yelp-uri", &uri, NULL);
     if (uri == NULL)
         return;
 
+    gdkwin = gtk_widget_get_window (GTK_WIDGET (window));
+    if (gdkwin != NULL)
+        gdk_window_set_cursor (gdkwin,
+                               gdk_cursor_new_for_display (gdk_window_get_display (gdkwin),
+                                                           GDK_WATCH));
+
     doc_uri = yelp_uri_get_document_uri (uri);
     if (priv->doc_uri == NULL || !g_str_equal (priv->doc_uri, doc_uri)) {
         window_set_bookmarks (window, doc_uri);
@@ -1545,11 +1552,13 @@ hidden_entry_hide (YelpWindow  *window)
 {
     YelpWindowPrivate *priv = GET_PRIV (window);
 
-    gtk_container_remove (GTK_CONTAINER (priv->hbox),
-                          priv->align_hidden);
-    gtk_box_pack_start (GTK_BOX (priv->hbox),
-                        priv->align_location,
-                        TRUE, TRUE, 0);
+    if (gtk_widget_get_parent (priv->align_hidden) != NULL) {
+        gtk_container_remove (GTK_CONTAINER (priv->hbox),
+                              priv->align_hidden);
+        gtk_box_pack_start (GTK_BOX (priv->hbox),
+                            priv->align_location,
+                            TRUE, TRUE, 0);
+    }
 }
 
 static gboolean
